@@ -74,23 +74,20 @@ class DBManager(object):
             pre_sql = 'insert into %s (%s) values(%s)  on duplicate key update %s' % (table, fields, pre_values, updates)
         else:
             pre_sql = 'insert into %s (%s) values(%s)' % (table, fields, pre_values)
-        session = None
-        bind_sql = None
         try:
             bind_sql = text(pre_sql)
-            session = self.engine.connect()
-            if type(data) == list:
-                resproxy = session.execute(bind_sql, *data)  # 对于数组，批量执行，效率非常高
-            else:
-                resproxy = session.execute(bind_sql, data)
-            return resproxy
+            with self.engine.connect() as session:
+                transaction = session.begin()
+                if type(data) == list:
+                    resproxy = session.execute(bind_sql, *data)  # 对于数组，批量执行，效率非常高
+                else:
+                    resproxy = session.execute(bind_sql, data)
+                transaction.commit()
+                return resproxy
         except Exception as e:
-            if bind_sql is not None:
-                self.engine.logger.error('插入insert操作\n%s\n发生错误\n%s' % (bind_sql, e))
+            transaction.rollback()
+            self.engine.logger.error(f'插入insert操作发生错误\n{e}')
             return None
-        finally:
-            if session:
-                session.close()
 
     def update(self, table, data):
         '''
@@ -115,6 +112,11 @@ class DBManager(object):
             if session:
                 session.close()
 
+    def query(self, sql):
+        result = self.execute(sql)
+        result = [dict(row) for row in result]
+        return result
+        
     def exists(self, table, fields, data):
         condition = ''
         if fields:
